/*---------------------------------------------------------------------------*\
  =========                 |
  \\      /  F ield         | OpenFOAM: The Open Source CFD Toolbox
   \\    /   O peration     |
    \\  /    A nd           | Copyright held by original authors.
     \\/     M anipulation  |
-------------------------------------------------------------------------------
License
    OpenFOAM is free software; you can redistribute it and/or modify it
    under the terms of the GNU General Public License as published by the
    Free Software Foundation; either version 2 of the License, or (at your
    option) any later version.

    OpenFOAM is distributed in the hope that it will be useful, but WITHOUT
    ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
    FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
    for more details.

    You should have received a copy of the GNU General Public License
    along with OpenFOAM; if not, write to the Free Software Foundation,
    Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA

Application
    potentialWaveFoam

Description
    Solves a linear wave propagation of velocity potential.

-------------------------------------------------------------------------------

Copyright (c) 2008 Takuya OSHIMA <oshima@eng.niigata-u.ac.jp>.
All rights reserved.

\*---------------------------------------------------------------------------*/

#define WRITE_PROBES 1

#include "fvCFD.H"
#include "OFstream.H"

// * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * //

int main(int argc, char *argv[])
{

// system header files
#   include "setRootCase.H"
#   include "createTime.H"
#   include "createMesh.H"
// User-defined header files
#   include "readTransportProperties.H"
#   include "createFields.H"
#if WRITE_PROBES
#   include "createAverages.H"
#endif

// * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * //

    Info<< "\nCalculating sound propagation\n" << endl;

#include "acousticCourantNo.H"

    dimensionedScalar c02 = c0 * c0;

    for (runTime++; !runTime.end(); runTime++)
    {
        Info<< "Time = " << runTime.timeName() << nl << endl;

	// Euler explicit
	solve(fvm::d2dt2(phi) == c02*fvc::laplacian(phi));
	phi.boundaryField().updateCoeffs();

#if WRITE_PROBES
	p = fvc::ddt(phi) * rho;
#include "writeAverages.H"
#endif

        runTime.write();

        Info<< "ExecutionTime = " << runTime.elapsedCpuTime() << " s"
            << "  ClockTime = " << runTime.elapsedClockTime() << " s"
            << nl << endl;
    }

    Info<< "End\n" << endl;

    return(0);
}


// ************************************************************************* //
